@@ -1,5 +1,6 @@
 #!/usr/bin/env python
-
+# -*- coding: utf-8 -*-
+#
 # Copyright 2012 GRNET S.A. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or
@@ -33,6 +34,10 @@
 # interpreted as representing official policies, either expressed
 # or implied, of GRNET S.A.
 
+"""This module is the entrance point for the non-interactive version of the
+snf-image-creator program.
+"""
+
 from image_creator import __version__ as version
 from image_creator.disk import Disk
 from image_creator.util import FatalError, MD5
@@ -44,6 +49,7 @@ import os
 import optparse
 import StringIO
 import signal
+import json
 
 
 def check_writable_dir(option, opt_str, value, parser):
@@ -77,12 +83,12 @@ def parse_options(input_args):
 
     parser.add_option("-u", "--upload", dest="upload", type="string",
                       default=False,
-                      help="upload the image to pithos with name FILENAME",
+                      help="upload the image to the cloud with name FILENAME",
                       metavar="FILENAME")
 
     parser.add_option("-r", "--register", dest="register", type="string",
                       default=False,
-                      help="register the image with ~okeanos as IMAGENAME",
+                      help="register the image with a cloud as IMAGENAME",
                       metavar="IMAGENAME")
 
     parser.add_option("-m", "--metadata", dest="metadata", default=[],
@@ -93,6 +99,15 @@ def parse_options(input_args):
                       default=None, help="use this authentication token when "
                       "uploading/registering images")
 
+    parser.add_option("-a", "--authentication-url", dest="url", type="string",
+                      default=None, help="use this authentication URL when "
+                      "uploading/registering images")
+
+    parser.add_option("-c", "--cloud", dest="cloud", type="string",
+                      default=None, help="use this saved cloud account to "
+                      "authenticate against a cloud when "
+                      "uploading/registering images")
+
     parser.add_option("--print-sysprep", dest="print_sysprep", default=False,
                       help="print the enabled and disabled system preparation "
                       "operations for this input media", action="store_true")
@@ -114,7 +129,7 @@ def parse_options(input_args):
                       help="don't shrink any partition", action="store_false")
 
     parser.add_option("--public", dest="public", default=False,
-                      help="register image with cyclades as public",
+                      help="register image with the cloud as public",
                       action="store_true")
 
     parser.add_option("--tmpdir", dest="tmp", type="string", default=None,
@@ -133,10 +148,13 @@ def parse_options(input_args):
     if options.register and not options.upload:
         raise FatalError("You also need to set -u when -r option is set")
 
-    if options.upload and options.token is None:
-        raise FatalError(
-            "Image uploading cannot be performed. "
-            "No authentication token is specified. Use -t to set a token")
+    if options.upload and (options.token is None or options.url is None) and \
+            options.cloud is None:
+
+        err = "You need to either specify an authentication URL and token " \
+              "pair or an available cloud name."
+
+        raise FatalError("Image uploading cannot be performed. %s" % err)
 
     if options.tmp is not None and not os.path.isdir(options.tmp):
         raise FatalError("The directory `%s' specified with --tmpdir is not "
@@ -184,13 +202,28 @@ def image_creator():
                 raise FatalError("Output file `%s' exists "
                                  "(use --force to overwrite it)." % filename)
 
-    # Check if the authentication token is valid. The earlier the better
-    if options.token is not None:
+    # Check if the authentication info is valid. The earlier the better
+    if options.token is not None and options.url is not None:
         try:
-            account = Kamaki.get_account(options.token)
+            account = Kamaki.create_account(options.url, options.token)
             if account is None:
-                raise FatalError("The authentication token you provided is not"
-                                 " valid!")
+                raise FatalError("The authentication token and/or URL you "
+                                 "provided is not valid!")
+            else:
+                kamaki = Kamaki(account, out)
+        except ClientError as e:
+            raise FatalError("Astakos client: %d %s" % (e.status, e.message))
+    elif options.cloud:
+        avail_clouds = Kamaki.get_clouds()
+        if options.cloud not in avail_clouds.keys():
+            raise FatalError(
+                "Cloud: `%s' does not exist.\n\nAvailable clouds:\n\n\t%s\n"
+                % (options.cloud, "\n\t".join(avail_clouds.keys())))
+        try:
+            account = Kamaki.get_account(options.cloud)
+            if account is None:
+                raise FatalError(
+                    "Cloud: `$s' exists but is not valid!" % options.cloud)
             else:
                 kamaki = Kamaki(account, out)
         except ClientError as e:
@@ -198,15 +231,16 @@ def image_creator():
 
     if options.upload and not options.force:
         if kamaki.object_exists(options.upload):
-            raise FatalError("Remote pithos object `%s' exists "
+            raise FatalError("Remote storage service object: `%s' exists "
                              "(use --force to overwrite it)." % options.upload)
         if kamaki.object_exists("%s.md5sum" % options.upload):
-            raise FatalError("Remote pithos object `%s.md5sum' exists "
-                             "(use --force to overwrite it)." % options.upload)
+            raise FatalError("Remote storage service object: `%s.md5sum' "
+                             "exists (use --force to overwrite it)." %
+                             options.upload)
 
     if options.register and not options.force:
         if kamaki.object_exists("%s.meta" % options.upload):
-            raise FatalError("Remote pithos object `%s.meta' exists "
+            raise FatalError("Remote storage service object `%s.meta' exists "
                              "(use --force to overwrite it)." % options.upload)
 
     disk = Disk(options.source, out, options.tmp)
@@ -221,36 +255,23 @@ def image_creator():
 
         image = disk.get_image(snapshot)
 
-        # If no customization is to be done, the image should be mounted ro
-        ro = (not (options.sysprep or options.shrink) or options.print_sysprep)
-        image.mount(ro)
-        try:
-            for sysprep in options.disabled_syspreps:
-                image.os.disable_sysprep(image.os.get_sysprep_by_name(sysprep))
+        for sysprep in options.disabled_syspreps:
+            image.os.disable_sysprep(image.os.get_sysprep_by_name(sysprep))
 
-            for sysprep in options.enabled_syspreps:
-                image.os.enable_sysprep(image.os.get_sysprep_by_name(sysprep))
+        for sysprep in options.enabled_syspreps:
+            image.os.enable_sysprep(image.os.get_sysprep_by_name(sysprep))
 
-            if options.print_sysprep:
-                image.os.print_syspreps()
-                out.output()
+        if options.print_sysprep:
+            image.os.print_syspreps()
+            out.output()
 
-            if options.outfile is None and not options.upload:
-                return 0
+        if options.outfile is None and not options.upload:
+            return 0
 
-            if options.sysprep:
-                err_msg = "Unable to perform the system preparation tasks. " \
-                    "Couldn't mount the media%s. Use --no-sysprep if you " \
-                    "don't won't to perform any system preparation task."
-                if not image.mounted:
-                    raise FatalError(err_msg % "")
-                elif image.mounted_ro:
-                    raise FatalError(err_msg % " read-write")
-                image.os.do_sysprep()
+        if options.sysprep:
+            image.os.do_sysprep()
 
-            metadata = image.os.meta
-        finally:
-            image.umount()
+        metadata = image.os.meta
 
         size = options.shrink and image.shrink() or image.size
         metadata.update(image.meta)
@@ -261,9 +282,9 @@ def image_creator():
         md5 = MD5(out)
         checksum = md5.compute(image.device, size)
 
-        metastring = '\n'.join(
-            ['%s=%s' % (key, value) for (key, value) in metadata.items()])
-        metastring += '\n'
+        metastring = unicode(json.dumps(
+            {'properties': metadata,
+             'disk-format': 'diskdump'}, ensure_ascii=False))
 
         if options.outfile is not None:
             image.dump(options.outfile)
@@ -286,7 +307,7 @@ def image_creator():
         try:
             uploaded_obj = ""
             if options.upload:
-                out.output("Uploading image to pithos:")
+                out.output("Uploading image to the storage service:")
                 with open(snapshot, 'rb') as f:
                     uploaded_obj = kamaki.upload(
                         f, size, options.upload,
@@ -303,12 +324,13 @@ def image_creator():
 
             if options.register:
                 img_type = 'public' if options.public else 'private'
-                out.output('Registering %s image with ~okeanos ...' % img_type,
-                           False)
-                kamaki.register(options.register, uploaded_obj, metadata,
-                                options.public)
+                out.output('Registering %s image with the compute service ...'
+                           % img_type, False)
+                result = kamaki.register(options.register, uploaded_obj,
+                                         metadata, options.public)
                 out.success('done')
                 out.output("Uploading metadata file ...", False)
+                metastring = unicode(json.dumps(result, ensure_ascii=False))
                 kamaki.upload(StringIO.StringIO(metastring),
                               size=len(metastring),
                               remote_path="%s.%s" % (options.upload, 'meta'))
@@ -323,7 +345,7 @@ def image_creator():
 
                 out.output()
         except ClientError as e:
-            raise FatalError("Pithos client: %d %s" % (e.status, e.message))
+            raise FatalError("Service client: %d %s" % (e.status, e.message))
 
     finally:
         out.output('cleaning up ...')
